#include "comdef.h"
#include "int-flash.h"

/* Chip-specific internal flash API header files */
#if defined(F28335)
  #include "../f28api/inc/Flash2833x_API_Library.h"
#else
  #include "../f28api/inc/Flash280x_API_Library.h"
#endif


FLASH_ST FlashStatus;

uint16 perform_int_flash_program() 
{
uint32 addr;
uint16 i;

#if defined(__TMS320C28XX__)
  /* Address must be aligned on 4-byte boundary */
  if ( address & 1 ) {
    status = STATUS_ADDRESS_ALIGN;
    return 0; /* return ERROR */
  }
  if ( length & 1 ) {
    status = STATUS_LENGTH_ALIGN;
    return 0; /* return ERROR */
  }

  // Check if non-FFFFFFFF keys (both, MSM & flash write protection)
  //   programming requested and not enabled //

  if (fProgKeys != 0x12345678) {

    // find address of keys (last 8 words of SECTOR_A
	addr = 0;
	for (i=0; i<sectors;i++) {
	  if (!FLASH_BANK(sector_def[i])) {
	    addr += (SECTOR_SIZE(sector_def[i])-16);
		break;
      }
      addr += (SECTOR_SIZE(sector_def[i]));
    }

	if (i>=sectors) {
      status = STATUS_INVALID_PARAMETER;
      return 0; // return ERROR//  
	}

    for (i=0; i<8; i++) { // 8 16-bit KEYS //
	  if ((address <= addr) && ((address + length) > addr))
	    if (0xFFFF != ((uint16*)buffer)[(addr-address)>>1]) {
          status = STATUS_KEYS_REPROGRAMMING;
          return 0; // return ERROR //
		}
	  addr += 2;
	}
  }

  if(Flash_Program( (uint16*)((address>>1) + base_address),
  					(uint16*)buffer,
  					 length>>1,
  					 &FlashStatus)!=0)
  {
    status = STATUS_FLASH_PROGRAM_FAIL;
    return 0; // return ERROR //
  }

  if(Flash_Verify((uint16*)((address>>1) + base_address),
  				  (uint16*)buffer,
  				  length>>1,
  				  &FlashStatus)!=0)
  {
    status = STATUS_VERIFY_ERR;
    return 0; // return ERROR //
  } 

  address += length; 
             
  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

uint16 perform_int_flash_erase()
{

#if defined(__TMS320C28XX__)
  uint16 sector;
  uint16 tmp = 0xFFFF;
  uint32 sector_addr = 0;

  /* find sector & bank for address */
  for (sector=0; sector < sectors; sector++) {
    if ( address < (sector_addr + SECTOR_SIZE(sector_def[sector]))) {
      /* sector found */
	  tmp = FLASH_BANK(sector_def[sector]);
	  break;
	}
	sector_addr += SECTOR_SIZE(sector_def[sector]);
  }

  if (tmp == 0xFFFF) {
    status = STATUS_INVALID_PARAMETER;
    return 0; // return ERROR//  
  }

  sector = 1 << tmp;

  if(Flash_Erase(sector,&FlashStatus)!=0){
    status = STATUS_FLASH_ERASE_FAIL;
    return 0; // return ERROR //  
  }
  return 1;
#else
  status = STATUS_NOT_IMPLEMENTED;
  return 0; /* return ERROR */
#endif
}

